package com.robotsfx.wimaster_testapp;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

public class MainActivity extends Activity {
	//メンバ
	private final static String TAG="WiMaster_testApp";
	private final static String IP="1.2.3.4"; 	//wimaster IPアドレス(固定)
	private final static int PORT=2000;			//wimaster PORT番号(固定)
	private final static int WifiComWhat = 1;	//ハンドラの識別用定数
	private Socket		 socket; 				//ソケット
	private InputStream  in;					//入力ストリーム
	private OutputStream out;					//出力ストリーム
	private static TextView statusText;			//センサー値表示用テキスト
	private SeekBar seekBar1;					//サーボ駆動用シークバー
	private Timer timer;						//センサー値を周期的に要求するためのタイマー
	
	/////////////////////////////////////////////////
	//	起動時処理
	/////////////////////////////////////////////////
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		//---UI---
		statusText = (TextView)findViewById(R.id.status_text);
		seekBar1 = (SeekBar)findViewById(R.id.seekBar_1);
		seekBar1.setOnSeekBarChangeListener(new SeekBarChngeListener());
		
		//---Timer---
        timer = new Timer();
        timer.schedule( new TimerTask(){
        	@Override
        	public void run(){
        		handler.post(new Runnable(){
        			public void run(){
        				timerInterval();		//インターバル時の動作
        			}
        		});
        	}
        }, 1000, 100);		//100mS毎に実行
		
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	////////////////////////////////////////////////
    //	onResume
	////////////////////////////////////////////////
    @Override
    public void onResume(){
    	super.onResume();	
    	//スレッド生成   	
    	Thread thread = new Thread(){
    		public void run(){
    			try{
    				connect(IP, PORT);		//resume時にソケットを開く
    			}catch(Exception e){
    				e.printStackTrace();
    			}
    		}
    	};
    	thread.start();
    } 
    
	////////////////////////////////////////////////
    //	onPause
	////////////////////////////////////////////////
    @Override
    public void onPause(){
    	//pause時にソケットを閉じる
    	try{
    		socket.close();
    		socket=null;
    	}catch(Exception e){	
    	}
    	super.onPause();
    }
    
	/////////////////////////////////////////////////
    //	接続
	/////////////////////////////////////////////////
    private void connect(String ip,int port) {
        Log.v(TAG,"connect start");
        int size;
        byte[] buffer=new byte[1024];
        try {
            //ソケット接続
            socket=new Socket(ip,port);            
            if( socket.isConnected() && socket != null)
            {
                in =socket.getInputStream();
                out=socket.getOutputStream();
                Log.v("connect","接続成功");
            }
            else
            {
                Log.v("connect","接続失敗");
            }
            //受信ループ
            while( socket.isConnected() && socket != null){
            	size = in.read(buffer);
            	if(size<=0) continue;
    			//データをハンドラに渡す
    			byte[] data = new byte[size];
    			System.arraycopy(buffer, 0, data, 0, size);
    			Log.d(TAG,"data length="+data.length);
				handler.obtainMessage(WifiComWhat, data).sendToTarget();
            }
            
        } catch (Exception e) {
            Log.v("connect", e.toString() );
        }
    }
    
	/////////////////////////////////////////////////
	//	シークバー変化時イベント
	/////////////////////////////////////////////////
	class SeekBarChngeListener implements OnSeekBarChangeListener{
		@Override 
		public void onStartTrackingTouch(SeekBar sb){
		}
		
		@Override 
		public void onStopTrackingTouch(SeekBar sb){
		}
		
		//シークバー変化時処理
		@Override 
		public void onProgressChanged(SeekBar sb,
				int progress, boolean fromTouch) {
			setServoPos(0, (byte)progress, 1);		//ID=3,#0サーボをprogressの位置に速度1で回転
		}
	}
    
	/////////////////////////////////////////////////
	//	サーボ位置変更
	/////////////////////////////////////////////////
	private void setServoPos(int num, int pos, int speed){
		try{
			byte[] sndData = new byte[7];
			sndData[0] = (byte)255;		//sync
			sndData[1] = (byte)3;		//board id(AGB65-RSCのデフォルト値)
			sndData[2] = (byte)4;		//data length
			sndData[3] = (byte)2;		//command
			sndData[4] = (byte)num;		//servo number
			sndData[5] = (byte)pos;		//servo position
			sndData[6] = (byte)speed;	//servo speed
			sendData(sndData);
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
    ////////////////////////////////////////////////
    //	データ送信
	////////////////////////////////////////////////	
    private void sendData(byte[] data){
    	try{
			//データの送信
			if(socket!=null && socket.isConnected()){
				out.write(data);
				out.flush();
			}
		}catch(Exception e){
			Log.d(TAG,"送信！！失敗しました");
			e.printStackTrace();
		}
    }
    
    ////////////////////////////////////////////////
    //	タイマーインターバル(100mS毎)
	////////////////////////////////////////////////
    private void timerInterval(){
    	requestAllSensorValue();
    }
    
    ////////////////////////////////////////////////
    //	センサー値要求
	////////////////////////////////////////////////	
    private void requestAllSensorValue(){
		try{
			byte[] sndData = new byte[4];
			sndData[0] = (byte)255;		//sync
			sndData[1] = (byte)120;		//board id
			sndData[2] = (byte)1;		//data length
			sndData[3] = (byte)1;		//command
			sendData(sndData);
		}catch(Exception e){
			e.printStackTrace();
		}
	}
    
    
    
    ////////////////////////////////////////////////
    //	イベントハンドラ
	////////////////////////////////////////////////	
    private static Handler handler = new Handler() {
    	@Override
    	public void handleMessage(Message msg) {
			switch(msg.what)
			{	
			case WifiComWhat:
				byte[] data = (byte[])msg.obj;
				//受信データdebug表示
				String str = "";
				for(int i=0; i<data.length; i++){
					str += String.valueOf(data[i]) + ",";
				}
				Log.d(TAG,"data="+str);
				chkRcvData(data);
				break;
			default:
				break;
			}	//switch
    	} //handleMessage
    }; //handler
    
    
    ////////////////////////////////////////////////
    //	受信データの解析と表示（ポート＃０のセンサーのみ値を表示）
	////////////////////////////////////////////////
    private static void chkRcvData(byte[] data){
    	byte[] sensorValue = new byte[16];		//#0-15のセンサー値を格納
    	//受信ヘッダ確認（センサーボードからの応答か？）
    	if(data[0]==(byte)255 && data[1]==(byte)120){
    		//値部分だけ抽出
    		for(int i=0; i<16; i++){
    			sensorValue[i] = data[4+i];
    		}
    		//センサーポート＃３の値を表示
    		String str = String.valueOf(convByteToInt(sensorValue[2]));	//*1
    		statusText.setText(str);
    	}
    }
    //*1JAVAでは8bit値は符号無しのみなので、127以上はマイナスの値に表示されてしまう。よってこれを符号無し32bitのintへ変換している。
    //センサーのポート番号は＃１～１６、byte配列は０～１５なので＃３の値はsensorValue[2]になる。
    
    ///////////////////////////////////////////////
    //	byteを符号無しintへ変換
	////////////////////////////////////////////////
    private static int convByteToInt(byte b){
		int i = (int)b;
		i &= 0x000000FF;
		return i;
	}
    
}
